package handlers

import (
	"errors"
	"fmt"
	"github.com/gofiber/fiber/v3"
	"lmika.dev/lmika/hugo-cms/models"
	"lmika.dev/lmika/hugo-cms/services/pages"
	"net/http"
)

type Pages struct {
	Svc *pages.Service
}

func (h *Pages) Index(c fiber.Ctx) error {
	site := GetSite(c)

	pages, err := h.Svc.ListPagesOfSite(c.Context(), site)
	if err != nil {
		return err
	}

	return c.Render("pages/index", fiber.Map{
		"pages": pages,
	}, "layouts/site")
}

func (h *Pages) New(c fiber.Ctx) error {
	return c.Render("pages/edit", fiber.Map{
		"page": models.Page{},
	}, "layouts/site")
}

func (h *Pages) Create(c fiber.Ctx) error {
	site := GetSite(c)

	var req struct {
		Title string `json:"title" form:"title"`
		Body  string `json:"body" form:"body"`
	}
	if err := c.Bind().Body(&req); err != nil {
		return err
	}

	_, err := h.Svc.Create(c.Context(), site, pages.NewPost{
		Title: req.Title,
		Body:  req.Body,
	})
	if err != nil {
		return err
	}

	return c.Redirect().To(fmt.Sprintf("/sites/%v/pages", site.ID))
}

func (h *Pages) Edit(c fiber.Ctx) error {
	site := GetSite(c)

	pageID := fiber.Params[int](c, "pageId")
	if pageID == 0 {
		return errors.New("pageId is required")
	}

	page, err := h.Svc.GetPage(c.Context(), pageID)
	if err != nil {
		return err
	} else if page.SiteID != site.ID {
		return fmt.Errorf("page id %v not equal to site id %v", pageID, site.ID)
	}

	return c.Render("pages/edit", fiber.Map{
		"page": page,
	}, "layouts/site")
}

func (h *Pages) Update(c fiber.Ctx) error {
	site := GetSite(c)

	pageID := fiber.Params[int](c, "pageId")
	if pageID == 0 {
		return errors.New("pageId is required")
	}

	var req struct {
		Title string `json:"title" form:"title"`
		Body  string `json:"body" form:"body"`
	}
	if err := c.Bind().Body(&req); err != nil {
		return err
	}

	if _, err := h.Svc.Update(c.Context(), site, int64(pageID), pages.NewPost{
		Title: req.Title,
		Body:  req.Body,
	}); err != nil {
		return err
	}

	return c.Redirect().To(fmt.Sprintf("/sites/%v/pages", site.ID))
}

func (h *Pages) Delete(c fiber.Ctx) error {
	site := GetSite(c)

	pageID := fiber.Params[int](c, "pageId")
	if pageID == 0 {
		return errors.New("pageID is required")
	}

	if err := h.Svc.DeletePage(c.Context(), site, pageID); err != nil {
		return err
	}

	return Select(c,
		HTMX(func(c fiber.Ctx) error {
			return c.Status(http.StatusOK).SendString("")
		}),
		Otherwise(func(c fiber.Ctx) error {
			return c.Redirect().To(fmt.Sprintf("/sites/%v/pages", site.ID))
		}),
	)
}
