package handlers

import (
	"bufio"
	"errors"
	"fmt"
	"github.com/gofiber/fiber/v3"
	"github.com/jackc/pgx/v5"
	"lmika.dev/lmika/hugo-cms/models"
	"lmika.dev/lmika/hugo-cms/providers/bus"
	"lmika.dev/lmika/hugo-cms/services/sites"
	"net/http"
	"time"
)

type Site struct {
	Site *sites.Service
	Bus  *bus.Bus
}

func (s *Site) Create(c fiber.Ctx) error {
	user := GetUser(c)

	site, err := s.Site.CreateSite(c.Context(), user, "New Site "+time.Now().Format("2006-01-02 15:04:05"))
	if err != nil {
		return err
	}

	UpdatePrefCookie(c, func(prefs *models.PrefCookie) {
		prefs.SiteID = site.ID
	})

	return c.Redirect().To(fmt.Sprintf("/sites/%v/posts", site.ID))
}

func (s *Site) Show(c fiber.Ctx) error {
	id := fiber.Params[int](c, "siteId")
	if id == 0 {
		return errors.New("siteId is required")
	}

	site, err := s.Site.GetSite(c.Context(), id)
	if err != nil {
		return err
	}

	return c.Render("sites/index", fiber.Map{
		"site": site,
	}, "layouts/main")
}

func (s *Site) Settings(c fiber.Ctx) error {
	return c.Render("sites/settings", fiber.Map{
		"themes": s.Site.Themes(),
	}, "layouts/site")
}

func (s *Site) SaveSettings(c fiber.Ctx) error {
	site := GetSite(c)

	var req sites.NewSettings
	if err := c.Bind().Body(&req); err != nil {
		return err
	}

	if err := s.Site.SaveSettings(c.Context(), site, req); err != nil {
		return err
	}

	return c.Redirect().To(fmt.Sprintf("/sites/%v/settings", site.ID))
}

func (s *Site) Rebuild(c fiber.Ctx) error {
	if err := s.Site.Rebuild(c.Context(), GetSite(c)); err != nil {
		return err
	}

	return c.Redirect().To(fmt.Sprintf("/sites/%v/posts", GetSite(c).ID))
}

func (s *Site) SSE(c fiber.Ctx) error {
	siteOfInterest := GetSite(c)

	c.Set("Content-Type", "text/event-stream")
	c.Set("Cache-Control", "no-cache")
	c.Set("Connection", "keep-alive")
	c.Set("Transfer-Encoding", "chunked")

	return c.SendStreamWriter(func(w *bufio.Writer) {
		sub := s.Bus.Subscribe()
		defer s.Bus.Unsubscribe(sub)

		for e := range sub.C {
			switch e.Type {
			case models.EventSiteBuildingStart:
				eventSite := e.Data.(models.Site)
				if eventSite.ID == siteOfInterest.ID {
					fmt.Fprintf(w, "event: site-build-status\n")
					fmt.Fprintf(w, "data: Building\n")
				}
			case models.EventSiteBuildingDone:
				eventSite := e.Data.(models.Site)
				if eventSite.ID == siteOfInterest.ID {
					fmt.Fprintf(w, "event: site-build-status\n")
					fmt.Fprintf(w, "data: \n")
				}
			}

			if err := w.Flush(); err != nil {
				break
			}
		}
	})
}

func (s *Site) WithSite() fiber.Handler {
	return func(c fiber.Ctx) (err error) {
		id := fiber.Params[int](c, "siteId")
		if id == 0 {
			return errors.New("siteId is required")
		}

		site, err := s.Site.GetSite(c.Context(), id)
		if err != nil {
			return err
		}

		user := GetUser(c)
		if site.OwnerUserID != user.ID {
			return c.Status(http.StatusForbidden).SendString("not permitted")
		}

		c.Locals("site", site)

		if prodTarget, err := s.Site.GetProdTargetOfSite(c.Context(), int(site.ID)); err == nil {
			c.Locals("prodTarget", prodTarget)
		} else if !errors.Is(err, pgx.ErrNoRows) {
			return err
		}

		return c.Next()
	}
}
