package sitebuilder

import (
	"context"
	"errors"
	"gopkg.in/yaml.v3"
	"lmika.dev/lmika/hugo-cms/models"
	"lmika.dev/lmika/hugo-cms/providers/hugo"
	"os"
	"path/filepath"
	"time"
)

func (s *Service) WritePost(site models.Site, post models.Post) models.Job {
	return models.Job{
		Do: func(ctx context.Context) error {
			s.signalSiteBuildingStarted(ctx, site)
			defer s.signalSiteBuildingFinished(ctx, site)

			rbn, err := s.fullRebuildNecessary(ctx, site)
			if err != nil {
				return err
			} else if rbn {
				return s.rebuildSite(ctx, site, site)
			}

			if err := s.writePost(site, post); err != nil {
				return err
			}
			return s.publish(ctx, site)
		},
	}
}

func (s *Service) WriteAllPosts(site models.Site) models.Job {
	return models.Job{
		Do: func(ctx context.Context) error {
			s.signalSiteBuildingStarted(ctx, site)
			defer s.signalSiteBuildingFinished(ctx, site)

			if err := s.writeAllPosts(ctx, site); err != nil {
				return err
			}

			return s.publish(ctx, site)
		},
	}
}

func (s *Service) DeletePost(site models.Site, post models.Post) models.Job {
	return models.Job{
		Do: func(ctx context.Context) error {
			s.signalSiteBuildingStarted(ctx, site)
			defer s.signalSiteBuildingFinished(ctx, site)

			themeMeta, ok := s.themes.Lookup(site.Theme)
			if !ok {
				return errors.New("theme not found")
			}

			postFilename := s.postFilename(site, themeMeta, post)

			if _, err := os.Stat(postFilename); err != nil {
				if errors.Is(err, os.ErrNotExist) {
					return nil
				}
				return err
			}

			if os.Remove(postFilename) != nil {
				return nil
			}
			return s.publish(ctx, site)
		},
	}
}

func (s *Service) writeAllPosts(ctx context.Context, site models.Site) error {
	var startId int64
	now := time.Now()
	for {
		posts, err := s.db.ListPublishablePosts(ctx, int64(startId), site.ID, now)
		if err != nil {
			return err
		} else if len(posts) == 0 {
			return nil
		}

		for _, post := range posts {
			if err := s.writePost(site, post); err != nil {
				return err
			}
		}
		startId = posts[len(posts)-1].ID
	}
}

func (s *Service) writePost(site models.Site, post models.Post) error {
	themeMeta, ok := s.themes.Lookup(site.Theme)
	if !ok {
		return errors.New("theme not found")
	}

	postFilename := s.postFilename(site, themeMeta, post)

	frontMatter := map[string]any{
		"date": post.PublishDate.Format(time.RFC3339),
	}
	if post.Title != "" {
		frontMatter["title"] = post.Title
	} else if themeMeta.PreferTitle {
		frontMatter["title"] = post.PublishDate.Format(time.ANSIC)
	}

	return s.writeMarkdownFile(postFilename, frontMatter, post.Body)
}

func (s *Service) writeMarkdownFile(outFile string, frontMatter map[string]any, body string) error {
	if err := os.MkdirAll(filepath.Dir(outFile), 0755); err != nil {
		return err
	}

	fmBytes, err := yaml.Marshal(frontMatter)
	if err != nil {
		return err
	}

	f, err := os.Create(outFile)
	if err != nil {
		return err
	}
	defer f.Close()

	if _, err := f.WriteString("---\n"); err != nil {
		return err
	}
	if _, err := f.Write(fmBytes); err != nil {
		return err
	}
	if _, err := f.WriteString("---\n"); err != nil {
		return err
	}
	if _, err := f.WriteString(body); err != nil {
		return err
	}

	return nil
}

func (s *Service) postFilename(site models.Site, themeMeta models.ThemeMeta, post models.Post) string {
	return filepath.Join(s.hugo.SiteStagingDir(site, hugo.ContentSiteDir), themeMeta.BlogPostBundle, post.CreatedAt.Format("2006-01-02-150405.md"))
}

func (s *Service) pageFilename(bi pageBuildInfo, page models.Page) string {
	isIndex := false
	isLeafBundle := true

	thisBundleInfo := bi.bundleInfo[bi.bundle.ID]
	if thisBundleInfo.PageCount > 1 || bi.bundle.Name == models.RootBundleName {
		isLeafBundle = false
		isIndex = thisBundleInfo.IndexPageID == page.ID
	} else {
		isIndex = true
	}

	bundleDir := ""
	if bi.bundle.Name != models.RootBundleName {
		bundleDir = bi.bundle.Name
	}

	pageName := page.Name
	if isIndex {
		if isLeafBundle {
			pageName = "index"
		} else {
			pageName = "_index"
		}
	}

	pageName += ".md"
	return filepath.Join(s.hugo.SiteStagingDir(bi.site, hugo.ContentSiteDir), bundleDir, pageName)
}
